package com.y_ys.bench.httpclient ;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;

/**
 * HttpClientAccess.
 * 
 * @version 2008/11/16
 * @author masahito suzuki
 * @since MBench 1.00
 */
public class HttpAccess {
    
    /**
     * HttpAccessBean.
     */
    private HttpAccessBean bean = null ;
    
    /**
     * HTTPクライアント.
     */
    private HttpClient client = null ;
    
    /**
     * HttpMethod.
     */
    private HttpMethod method = null ;
    
    
    /** コネクションBean **/
    private static class HttpAccessBean {
        public boolean method ;
        public HttpHeader header ;
        public byte[] body ;
        public String url ;
        public int timeout ;
        public String userAgent ;
        protected HttpAccessBean( boolean method,HttpHeader header,byte[] body,String url,int timeout,String userAgent ) {
            this.method = method ;
            this.header = header ;
            this.body = body ;
            this.url = url ;
            this.timeout = timeout ;
            this.userAgent = userAgent ;
        }
    }
    
    /**
     * コンストラクタ.
     */
    private HttpAccess() {
    }
    
    /**
     * コンストラクタ.
     * @param method コネクションメソッドタイプを設定します.<br>
     *               [true]がPOST,[false]がGETです.
     * @param header HTTPヘッダ情報を設定します.
     * @param body [post]の場合のBodyデータを設定します.
     * @param url 接続先URLを設定します.
     * @param timeout タイムアウト値をミリ秒単位で設定します.
     * @param userAgent 接続時のUserAgentを設定します.
     * @exception Exception 例外.
     */
    public HttpAccess( boolean method,HttpHeader header,byte[] body,String url,int timeout,String userAgent )
        throws Exception {
        this.bean = new HttpAccessBean( method,header,body,url,timeout,userAgent ) ;
        this.client =  new HttpClient() ;
        this.recreate() ;
    }
    
    protected void finalize() throws Exception {
        destroy() ;
    }
    
    /** 全体再生成. **/
    protected void recreate()
        throws Exception {
        this.client.getHttpConnectionManager().getParams().setLinger( 0 ) ;
        if( bean.timeout > 0 ) {
            this.client.getParams().setSoTimeout( bean.timeout ) ;
        }
        HttpMethod md = createMethod( bean.method,bean.header,bean.body,bean.url,bean.userAgent ) ;
        this.client.getParams().setAuthenticationPreemptive( false ) ;
        this.method = md ;
    }
    
    /** HttpClient再生成. **/
    protected void recreateByHttpClient() {
        this.client.getHttpConnectionManager().getParams().setLinger( 0 ) ;
        if( bean.timeout > 0 ) {
            this.client.getParams().setSoTimeout( bean.timeout ) ;
        }
        this.client.getParams().setAuthenticationPreemptive( false ) ;
    }
    
    /** HttpMethodを生成. **/
    private static final HttpMethod createMethod( boolean method,HttpHeader header,byte[] body,String url,String userAgent )
        throws Exception {
        boolean bodyFlag = false ;
        HttpMethod md = null ;
        if( method ) {
            md = new PostMethod( url ) ;
            if( body != null && body.length > 0 ) {
                bodyFlag = true ;
                ( ( PostMethod )md ).setRequestEntity( new ByteArrayRequestEntity( body ) ) ;
            }
        }
        else {
            md = new GetMethod( url ) ;
        }
        int len = ( header != null ) ? header.size() : 0 ;
        for( int i = 0 ; i < len ; i ++ ) {
            md.addRequestHeader( header.get( i ) ) ;
        }
        if( userAgent != null && ( userAgent = userAgent.trim() ).length() > 0 ) {
            md.addRequestHeader( "User-Agent",userAgent ) ;
        }
        if( bodyFlag ) {
            md.addRequestHeader( "Content-Type","application/x-www-form-urlencoded" ) ;
        }
        md.addRequestHeader( "Connection","keep-alive" ) ;
        md.setFollowRedirects( false ) ;
        md.getParams().setParameter(
            HttpMethodParams.RETRY_HANDLER,
            new DefaultHttpMethodRetryHandler( 3,false )
        ) ;
        md.setDoAuthentication( false ) ;
        return md ;
    }
    
    /**
     * オブジェクト破棄.
     */
    public void destroy() {
        if( method != null ) {
            try {
                method.releaseConnection() ;
            } catch( Exception e ) {
            }
            try {
                method.abort() ;
            } catch( Exception e ) {
            }
        }
        method = null ;
        client = null ;
        bean = null ;
    }
    
    /**
     * コネクションクローズ.
     * @exception Exception 例外.
     */
    public void close() throws Exception {
        if( method != null ) {
            try {
                method.releaseConnection() ;
            } catch( Exception e ) {
            }
            this.client =  new HttpClient() ;
            recreateByHttpClient() ;
        }
    }
    
    /**
     * コネクションリリース.
     */
    public void release() {
        if( method != null ) {
            try {
                method.releaseConnection() ;
            } catch( Exception e ) {
            }
        }
    }
    
    /**
     * HTTPClient実行.
     * @param result HttpResult取得用オブジェクトを設定します.
     * @return boolean [true]の場合、正常終了.
     * @exception Exception 例外.
     */
    public boolean execution( HttpResult result ) throws Exception {
        client.executeMethod( method ) ;
        return getHttpResponse( result,method ) ;
    }
    
    /** レスポンス取得. **/
    private static final boolean getHttpResponse( HttpResult result,HttpMethod method )
        throws Exception {
        try {
            int state = method.getStatusCode() ;
            // 正常取得.
            if( state == 200 ) {
                byte[] binary = null ;
                InputStream in = method.getResponseBodyAsStream() ;
                ByteArrayOutputStream out = new ByteArrayOutputStream() ;
                try {
                    byte[] buf = null ;
                    
                    buf = new byte[8192] ;
                    int n ;
                    while ((n = in.read(buf)) >= 0) {
                        if (n > 0) {
                            out.write(buf, 0, n) ;
                        }
                    }
                    out.flush() ;
                    binary = out.toByteArray() ;
                    out.close() ;
                    out = null ;
                    in.close() ;
                    in = null ;
                } finally {
                    if( in != null ) {
                        try {
                            in.close() ;
                        } catch( Exception e ) {
                        }
                    }
                    if( out != null ) {
                        try {
                            out.close() ;
                        } catch( Exception e ) {
                        }
                    }
                }
                Header contentType = method.getResponseHeader( "Content-Type" ) ;
                Header connection = method.getResponseHeader( "Connection" ) ;
                Header keepAlive = method.getResponseHeader( "Keep-Alive" ) ;
                result.clear() ;
                result.setContentType( ( contentType == null ) ? null : contentType.getValue() ) ;
                result.setConnection( ( connection == null ) ? null : connection.getValue() ) ;
                result.setKeepAlive( ( keepAlive == null ) ? null : keepAlive.getValue() ) ;
                result.setState( state ) ;
                result.setBody( binary ) ;
                //System.out.println( "result:" + result ) ;
                return true ;
            }
            // エラー取得.
            else {
                result.clear() ;
                result.setState( state ) ;
                return false ;
            }
        } finally {
            if( method != null ) {
                try {
                    method.releaseConnection() ;
                } catch( Exception e ) {
                }
            }
        }
    }
    
    /**
     * メソッドタイプを取得.
     * @return boolean [true]の場合POSTです.
     */
    public boolean isType() {
        return bean.method ;
    }
}
